'use strict';

/**
 * Gestion du diaporama.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
function Diaporama(q, p, o)
{
	// Indique si la lecture automatique est en cours.
	var _autoActive = false;

	// Indique si le bouton de la souris est enfoncé
	// pour la modification de la durée d'affichage
	// des fichiers en lecture automatique.
	var _autoDurationMouseDown = false;

	// Indique si un message du mode automatique est affiché.
	var _autoMsgActive = false;

	// Timer pour l'affichage des textes du mode automatique.
	var _autoMsgTimer;

	// Timer pour le changement de fichiers en lecture automatique.
	var _autoTimer;

	// Indique si une animation du carrousel est en cours.
	var _carouselAnimation = false;

	// Numéro de la 'page' courante du carrousel.
	var _carouselCurrentPage = 0;

	// Position courante dans le carrousel.
	var _carouselCurrentPosition = 1;

	// Nombre maximal de vignettes à afficher dans le carrousel.
	var _carouselMaxThumbs = 0;

	// Plus grande position des fichiers du carrousel.
	var _carouselMaxPosition = 0;

	// Nombre de pages du carrousel.
	var _carouselPagesCount = 0;

	// Informations utiles des fichiers du carrousel.
	var _carouselThumbs;

	// Indique si une animation sur les barres de contrôle est en cours.
	var _controlBarsAnimation = false;

	// Le pointeur de la souris se trouve-t-il sur une barre de contrôle ?
	var _controlBarsOver = false;

	// Position du fichier courant dans la série.
	var _currentPosition = 1;

	// Contrôle au clavier actif ?
	var _keyboardActive = true;

	// Verrou utilisé lorsqu'un effet de transition entre fichiers est en cours.
	var _itemTransitionLock = false;

	// Le diaporama a-t-il déjà été initialisé ?
	var _init = false;

	// Informations des fichiers.
	var _items = {};

	// Nombre de fichiers dans la série.
	var _itemsCount = 0;

	// Options.
	var _options =
	{
		// Activer les animations ?
		animate: true,

		// Durée d'affichage par défaut d'un fichier en lecture automatique.
		// En secondes.
		autoDuration: 3.0,

		// Durée d'affichage minimale des fichiers en lecture automatique.
		// En secondes.
		autoDurationMin: 1,

		// Durée d'affichage maximale des fichiers en lecture automatique.
		// En secondes.
		autoDurationMax: 60,

		// Précision pour la durée d'affichage des fichiers en lecture automatique.
		// En secondes.
		autoDurationPrecision: 0.5,

		// En lecture automatique, recommencer la lecture depuis
		// le premier fichier lorsqu'on est arrivé au dernier ?
		autoLoop: false,

		// Démarrer la lecture automatique au lancement du diaporama ?
		autoStart: false,

		// Afficher par défaut le carrousel ?
		carousel: true,

		// Épaisseur de la bordure du haut du carrousel.
		carouselBorderTop: 2,

		// Durée de l'effet de transition entre les pages du carrousel.
		carouselNavDuration: 600,

		// Marge externe des vignettes du carrousel, en pixels.
		carouselThumbsMargin: 8,

		// Épaisseur de la bordure des vignettes du carrousel, en pixels.
		carouselThumbsBorder: 1,

		// Dimensions des vignettes du carrousel, en pixels.
		// Valeur minimale : 50.
		carouselThumbsSize: 80,

		// Afficher les barres de contrôle ?
		controlBars: false,

		// Lancer le diaporama en mode plein écran ?
		fullScreen: false,

		// Lancer le diaporama en mode plein écran pour mobile ?
		fullScreenMobile: true,

		// Nombre de fichiers à récupérer lors des requêtes SQL
		// avant et après la position du fichier courant dans la série,
		// ainsi qu'au début et à la fin de la série.
		itemLimit: 10,

		// Nombre de fichiers à précharger autour du fichier courant.
		itemPreload: 2,

		// Durée de l'effet de redimensionnement du fichier
		// pour le switch taille réelle / taille redimensionnée.
		// 0 pour désactiver l'animation.
		// En millisecondes.
		itemResizeDuration: 250,

		// Autoriser le contrôle au clavier ?
		keyboard: true,

		// Largeur maximale jusqu'où l'on considère
		// un écran comme un écran mobile.
		mobileMaxWidth: 599,

		// Afficher la description par dessus les images (mais pas les vidéos) ?
		overImageDescription: false,

		// Afficher le titre par dessus les images (mais pas les vidéos) ?
		overImageTitle: false,

		// Afficher les informations au lancement du diaporama ?
		showInformations: false,

		// Parties à afficher dans le panneau d'informations.
		sidebarInformations: {u:1,d:1,s:1,p:1,t:1,e:1,i:1,x:1},

		// Le panneau latéral doit-il ne pas masquer le fichier ?
		sidebarItemResize: true,

		// Durée de l'effet d'apparition des panneaux latéraux.
		// 0 pour désactiver l'animation.
		// En millisecondes.
		sidebarShowDuration: 250,

		// Activer le swipe ?
		swipe: true,

		// Distance minimale à parcourir pour changer de fichier.
		// En pixels.
		swipeDistance: 30,

		// Durée de l'effet d'animation du swipe.
		// En millisecondes.
		swipeDuration: 200,

		// Durée de l'effet de transition entre fichiers par défaut.
		// En millisecondes.
		transitionDuration: 500,

		// Effet de transition entre fichiers par défaut.
		transitionEffect: 'fade',

		// Activer le zoom à la molette de la souris ?
		zoom: true,

		// Facteur multiplicateur à chaque étape du zoom.
		zoomFactor: 1.1,

		// Limite du zoom, en pourcentage.
		// 0 pour aucune limite.
		zoomLimit: 100,

		// Activer également le zoom pour les vidéos ?
		zoomVideo: true,

		// Zoomer dans la partie de l'image où se trouve le pointeur de la souris ?
		zoomZone: true
	};

	// Valeur CSS "overflow" pour <html> et <body>.
	var _overflow = {};

	// Précédentes dimensions du diaporama
	// (avant un changement de dimensions de la zone d'affichage).
	var _previousDiaporamaSize;

	// Le fichier courant est-il en taille réelle ?
	var _realsize = false;

	// Verrou utilisé lorsqu'un panneau latéral est en mouvement.
	var _sidebarMoveLock = false;

	// Indique si un swipe est en cours.
	var _swipeActive = false;

	// Paramètres du zoom.
	var _zoom = {};



	/**
	 * Démarre le diaporama.
	 *
	 * @return void
	 */
	this.start = function()
	{
		if ($('#diaporama').is(':visible'))
		{
			return;
		}

		// On supprime les barres de défilement du navigateur
		// et on désactive l'affichage de la galerie.
		_overflow =
		{
			html: $('html').css('overflow'),
			body: $('body').css('overflow'),
			scroll: window.scrollY
		};
		$('html,body').css('overflow', 'hidden');
		$('#gallery').hide();

		// Si le diaporama a déjà été chargé.
		if (_init)
		{
			if (_currentPosition != p)
			{
				_deleteItems(true);
				_currentPosition = p;
				_itemsCount = 0;
				_realsize = false;
				_getData(true, false, p, p);
				$('#diaporama_navigation_switch').removeClass('fullsize');
			}

			// Mode plein écran.
			_fullScreen();

			// On affiche le diaporama.
			$('#diaporama').show();

			_carouselChangeSizePosition();

			_changeCenterSizePosition();

			_keyboardActive = true;

			// Dimensions et position du fichier.
			_changeItemSizePosition(_currentPosition, false, true);

			return;
		}

		// Ajout du code HTML.
		if ($('#diaporama').length)
		{
			$('#diaporama').remove();
		}
		$('body').append(diaporama_html);

		// Options.
		_setOptions();

		// Mode plein écran.
		_fullScreen();

		// Redimensionnement du diaporama.
		$(window).resize(() => _resize());

		// Événements pour la lecture automatique.
		_setAutoEvents();

		// Événements sur les boutons.
		_setButtonsEvents();

		// Événements concernant le carrousel.
		_setCarouselEvents();

		// Gestion des favoris et de la sélection.
		_setFavoritesEvents();
		_setSelectionEvents();

		// Gestion des panneaux latéraux.
		_setSidebarsEvents();

		// Option pour afficher les informations par défaut.
		if (_options.showInformations && _options.controlBars)
		{
			setTimeout(() => _icon('informations').click(), 100);
		}

		// Gestion du swipe.
		_setSwipeEvents();

		// Gestion du zoom.
		_setZoomEvents();

		// Position.
		_currentPosition = p;

		// Récupération des données.
		_getData(true, false, p, p);

		// Durée d'affichage des fichiers en lecture automatique.
		_autoTextDuration();

		// On affiche le diaporama.
		$('#diaporama').show();

		// Gestion de l'affichage des barres de contrôle.
		_setControlBarsEvents();

		// Change les dimensions et la position du carrousel.
		_carouselChangeSizePosition();

		// Change la position de l'icône de chargement.
		_changeCenterSizePosition();

		// Dimensions actuelles du diaporama.
		_previousDiaporamaSize = _getDiaporamaSize();

		// Contrôle au clavier.
		_keyboard();

		// On indique que le diaporama a été initialisé.
		_init = true;
	};



	/**
	 * Génère une alerte pour afficher un message d'erreur.
	 *
	 * @param string message
	 *
	 * @return void
	 */
	function _alertError(message)
	{
		if (message == '')
		{
			message = 'Unknown error.';
		}
		else if (message.length > 500)
		{
			message = message.slice(0, 500) + '...';
		}
		alert('Error: ' + message);
	}

	/**
	 * Change la durée d'affichage des fichiers en lecture automatique.
	 *
	 * @param float i
	 *   Intervalle entre chaque palier de durée.
	 * @param integer d
	 *   Durée entre chaque palier de durée
	 *   lorsqu'on reste appuyé sur le bouton.
	 *
	 * @return void
	 */
	function _autoChangeDuration(i, d)
	{
		// Si le bouton n'est plus appuyé, on ne va pas plus loin.
		if (!_autoDurationMouseDown)
		{
			return;
		}

		// Accélération.
		d = !d ? 170 : (d > 20 ? d - 5 : d);

		// La durée doit-être comprise entre une valeur minimale et une valeur maximale.
		if (_options.autoDuration + i >= _options.autoDurationMin
		 && _options.autoDuration + i <= _options.autoDurationMax)
		{
			_options.autoDuration = Math.round((_options.autoDuration + i) * 10) / 10;

			_autoTextDuration();

			setTimeout(() => _autoChangeDuration(i, d), d);
		}
	}

	/**
	 * Ajoute au diaporama la durée d'affichage
	 * des fichiers lors de la lecture automatique.
	 *
	 * @return void
	 */
	function _autoTextDuration()
	{
		var duration = _options.autoDuration.toString();

		$('#diaporama_auto_duration_option input').val(duration);

		if (!duration.match(/\./))
		{
			duration += '.0';
		}
		$('#diaporama_auto_duration,#diaporama_auto_duration_option span').text(duration + ' s');

		if (_init && !$('#diaporama_bottom').is(':visible'))
		{
			_autoChangeMessage(duration);
		}
	}

	/**
	 * Gère le changement de fichiers durant la lecture automatique.
	 *
	 * @param bool start
	 *   Démarre-t-on la lecture automatique ?
	 *
	 * @return void
	 */
	function _autoChangeItem(start)
	{
		// Si la lecture automatique a été stoppée, on casse la boucle.
		if (!_autoActive)
		{
			return;
		}

		if (!start)
		{
			// Tant que l'on est pas au dernier fichier, on continue.
			if (_currentPosition != _itemsCount)
			{
				_changeItem('next');
			}

			// Si l'on est arrivé au dernier fichier, on recommence
			// au premier si l'option pour boucler est à true.
			else if (_options.autoLoop)
			{
				_changeItem('first');
			}
		}

		// On attend le chargement du fichier avant de passer au suivant.
		$('#diaporama_loading').show();
		$('#diaporama_item_' + _currentPosition)
			.one('load', function()
			{
				$('#diaporama_loading').hide();

				var transition_duration = _options.transitionEffect != 'none' && !start
					? parseInt(_options.transitionDuration)
					: 0;
				_autoTimer = setTimeout(
					() => _autoChangeItem(),
					(_options.autoDuration * 1000) + transition_duration
				);
			})
			.each(function()
			{
				if (this.complete || this.readyState == 4 || this.readyState == 'complete')
				{
					$(this).trigger('load');
				}
			});
	}

	/**
	 * Gestion de l'affichage des messages utilisés
	 * pour la lecture automatique.
	 *
	 * @param string text
	 *   Texte à afficher.
	 *
	 * @return void
	 */
	function _autoChangeMessage(text)
	{
		var msg = $('#diaporama_auto_message');

		msg.text(text);

		if (_autoMsgActive)
		{
			msg.stop(true, true);
			clearTimeout(_autoMsgTimer);
			_autoMsgTimer = undefined;
		}

		_autoMsgActive = true;

		msg.css('visibility', 'visible').fadeIn(1000, function()
		{
			_autoMsgTimer = setTimeout(() => msg.fadeOut(1000,
				() => _autoMsgActive = false), 2000);
		});
	}

	/**
	 * Change l'indicateur du fichier courant du caroussel.
	 *
	 * @return void
	 */
	function _carouselChangeCurrent()
	{
		if (!_options.carousel)
		{
			return;
		}

		$('#diaporama_carousel_thumbs dl').removeClass('current');
		$('#diaporama_carousel_image_' + _currentPosition).addClass('current');
	}

	/**
	 * Gestion des pages du carrousel.
	 *
	 * @return void
	 */
	function _carouselChangePages()
	{
		// Détermine le nombre de pages ainsi que la page courante.
		var i = Math.floor(_carouselMaxThumbs) - 1;
		_carouselPagesCount = Math.ceil((_itemsCount - 1) / i);
		_carouselCurrentPage = Math.ceil(_carouselCurrentPosition / i);
		if (_carouselCurrentPage > _carouselPagesCount)
		{
			_carouselCurrentPage = _carouselPagesCount
		}

		// Bouton précédent.
		if (_carouselCurrentPage > 1)
		{
			$('a[data-name="carousel_prev"]').removeClass('disabled');
		}
		else
		{
			$('a[data-name="carousel_prev"]').addClass('disabled');
		}

		// Bouton suivant.
		if (_carouselCurrentPage < _carouselPagesCount)
		{
			$('a[data-name="carousel_next"]').removeClass('disabled');
		}
		else
		{
			$('a[data-name="carousel_next"]').addClass('disabled');
		}
	}

	/**
	 * Change les dimensions et la position du carrousel.
	 *
	 * @param int width
	 *
	 * @return void
	 */
	function _carouselChangeSizePosition(width)
	{
		var s = _getDiaporamaSize();
		s.availableWidth += width ? width : 0;
		var next_icon = $('a[data-name="carousel_next"]');
		var prev_icon = $('a[data-name="carousel_prev"]');
		var next_css = {left: s.availableWidth - prev_icon.outerWidth()};

		var change = function()
		{
			// Détermine le nombre maximum de vignettes que peut contenir le carrousel.
			_carouselMaxThumbs = (s.availableWidth
				- (prev_icon.outerWidth() * 2)
				- _options.carouselThumbsMargin)
				/ (_options.carouselThumbsSize + (_options.carouselThumbsBorder * 2)
				+ _options.carouselThumbsMargin);

			// On regénère les images.
			_carouselMaxPosition = 0;
			_carouselChangeThumbs();
		};

		// Hauteur du carrousel.
		var height_add = (_options.carouselThumbsMargin * 2)
			+ (_options.carouselThumbsBorder * 2) + _options.carouselBorderTop;
		$('#diaporama_carousel').css({height: _options.carouselThumbsSize + height_add});

		// On crée une animation seulement lorsqu'un panneau latéral est affiché ou caché.
		if (width && _options.animate)
		{
			next_icon.animate(next_css, _options.sidebarShowDuration, () => change());
		}
		else
		{
			next_icon.css(next_css);
			change();
		}
	}

	/**
	 * Change les vignettes du carrousel.
	 *
	 * @param string nav
	 *
	 * @return void
	 */
	function _carouselChangeThumbs(nav)
	{
		if (_carouselThumbs === undefined)
		{
			return;
		}

		// Retourne la position à partir de laquelle on
		// doit afficher les vignettes dans le carrousel.
		var get_carousel_start = function()
		{
			var i = Math.floor(_carouselMaxThumbs) - 1;

			return ((Math.ceil(_carouselCurrentPosition / i) - 1) * i) + 1;
		};

		if (_currentPosition >= _carouselMaxPosition)
		{
			var center, width, height, html = '', n = 0;
			var thumbs_count = Math.floor(_carouselMaxThumbs) - 1;
			var max_thumbs = Math.ceil(_carouselMaxThumbs);

			// Position à partir de laquelle on affiche les vignettes.
			var start = get_carousel_start();
			if (start == _itemsCount)
			{
				start -= thumbs_count;
				if (start < 1)
				{
					start = 1;
				}
			}

			// Dans le cas d'une navigation dans le carrousel,
			// on supprimera les vignettes après l'animation.
			if (nav)
			{
				_carouselAnimation = true;
				_carouselCurrentPosition = start;

				var s = _getDiaporamaSize();
				var margin_left = thumbs_count * (_options.carouselThumbsSize
					+ (_options.carouselThumbsBorder * 2) + _options.carouselThumbsMargin);

				if (nav == 'prev')
				{
					$('#diaporama_carousel_thumbs dl:gt('
						+ (max_thumbs - thumbs_count) + ')').addClass('old');
				}
				else
				{
					$('#diaporama_carousel_thumbs dl:lt(' + thumbs_count + ')').addClass('old');
				}
			}

			// Sinon, on supprime tout de suite toutes les vignettes,
			// mais seulement si c'est nécessaire.
			else
			{
				var first_position = $('#diaporama_carousel_thumbs dl').length
					? parseInt($('#diaporama_carousel_thumbs dl:first').attr('id')
						.replace(/diaporama_carousel_image_(\d+)/, '$1'))
					: start;
				if (first_position != start)
				{
					$('#diaporama_carousel_thumbs').empty();
				}
			}

			// Création et insertion des nouvelles vignettes.
			for (var pos in _carouselThumbs)
			{
				if (pos < start || $('#diaporama_carousel_image_' + pos).is('dl'))
				{
					continue;
				}
				if (nav == 'prev' && n == thumbs_count)
				{
					break;
				}
				n++;

				var duration = _carouselThumbs[pos].is_video
					? '<span class="duration">' + _carouselThumbs[pos].duration + '</span>'
					: '';

				width = _options.carouselThumbsSize;
				height = _options.carouselThumbsSize;
				html += '<dl title="' + _carouselThumbs[pos].title
					+ '" id="diaporama_carousel_image_' + pos + '"><dt><a rel="' + n
					+ '" style="width:' + width + 'px;height:' + height + 'px;">' + duration
					+ '<img src="' + _carouselThumbs[pos].source + '"></a></dt></dl>' + "\n";

				_carouselMaxPosition = pos;

				if (n == max_thumbs)
				{
					break;
				}
			}
			if (nav == 'prev')
			{
				$('#diaporama_carousel_thumbs').css({marginLeft: -margin_left}).prepend(html);
			}
			else if (html !== '')
			{
				$('#diaporama_carousel_thumbs').append(html);
			}

			// Suppression des anciennes vignettes lors
			// de la navigation dans le carrousel.
			if (nav)
			{
				var nav_css = {marginLeft: nav == 'prev' ? 0 : -margin_left};

				$('#diaporama_carousel_thumbs').animate(
					nav_css,
					_options.animate ? _options.carouselNavDuration : 0,
					'swing',
					function()
					{
						$('#diaporama_carousel_thumbs dl.old').remove();
						$('#diaporama_carousel_thumbs').css('margin-left', 0);

						_getData(false, false, 0, _carouselCurrentPosition);
						_carouselAnimation = false;
					}
				);
			}

			// Gestion de l'événement "click" sur les vignettes du carrousel.
			$('#diaporama_carousel_thumbs dl').click(function()
			{
				var items_count = _itemsCount;
				var new_current_position = parseInt($(this).attr('id')
					.replace(/diaporama_carousel_image_(\d+)/, '$1'));

				if (new_current_position == _currentPosition)
				{
					return;
				}

				// On réinitialise le diaporama et on récupère les informations des images.
				_options.autoStart = false;
				_zoom = {};
				_reload(new_current_position);

				// Position du fichier courant dans le carrousel.
				_carouselChangeCurrent();

				// On regénère le carrousel.
				if ($(this).find('a').attr('rel') >= Math.floor(_carouselMaxThumbs))
				{
					_carouselMaxPosition = 0;
					_getData(false, false, 0, new_current_position);
				}

				// On réinitialise la lecture automatique.
				if (_autoActive)
				{
					if ((new_current_position == items_count) && !_options.autoLoop)
					{
						_icon('stop').click();
					}
					else
					{
						clearTimeout(_autoTimer);
						_autoTimer = setTimeout(
							() => _autoChangeItem(), _options.autoDuration * 1000
						);
					}
				}
			});

			// Change les pages du carrousel.
			_carouselChangePages();
		}

		// Position du fichier courant dans le carrousel.
		if (!nav)
		{
			_carouselChangeCurrent();
		}
	}

	/**
	 * Modifie les boutons de navigation.
	 *
	 * @return void
	 */
	function _changeButtonsNavigation()
	{
		var buttons =
		{
			first: 1,
			prev: _currentPosition <= 2 ? 1 : _currentPosition - 1,
			next: _currentPosition >= _itemsCount ? _itemsCount : _currentPosition + 1,
			last: _itemsCount
		};

		for (var button in buttons)
		{
			if (_items[buttons[button]] && _currentPosition != buttons[button])
			{
				$('a[data-name="' + button + '"]').removeClass('disabled');
			}
			else
			{
				$('a[data-name="' + button + '"]').addClass('disabled');
			}
		}
	}

	/**
	 * Change le bouton du switch taille réelle / taille redimensionnée du fichier.
	 *
	 * @param int width
	 * @param int height
	 *
	 * @return void
	 */
	function _changeButtonSwitch(width, height)
	{
		if (_items[_currentPosition] === undefined)
		{
			return;
		}

		var item = $('#diaporama_item_' + _currentPosition);
		item.unbind('mousedown');

		$('#diaporama_switch').removeClass('fullsize disabled');

		var s = _getDiaporamaSize();
		var item_width = width || item.width();
		var item_height = height || item.height();
		var exceeds = item_width > s.availableWidth || item_height > s.availableHeight;
		var zoom = item_width > _items[_currentPosition].width_resized;

		// Bouton de switch.
		if (zoom || exceeds)
		{
			$('#diaporama_switch').addClass('fullsize');
		}
		else if (item_width == _items[_currentPosition].width_resized)
		{
			$('#diaporama_switch').addClass('disabled');

			if (_zoom.width == item_width)
			{
				_realsize = false;
				_zoom = {};
			}
		}

		if (_zoom.width != item_width)
		{
			_zoom = {};
		}

		// On indique la nouvelle taille en pourcentage.
		_changePosition(item_width);

		// Si c'est une vidéo, on arrête ici.
		if (_items[_currentPosition].is_video)
		{
			return;
		}

		// Pointeur et rendu de l'image.
		item.css
		({
			'image-rendering': zoom ? 'pixelated' : 'auto',
			'cursor': exceeds ? 'move' : 'default'
		});

		// Déplacement de l'image.
		if (exceeds)
		{
			item.mousedown(function() { return _dragImage(this, event); });

			// Version mobile.
			document.getElementById('diaporama_item_' + _currentPosition)
				.addEventListener('touchstart', (e) => _dragImageTouch(e));
		}
	}

	/**
	 * Change la taille des conteneurs des éléments à centrer.
	 *
	 * @param int width
	 *
	 * @return void
	 */
	function _changeCenterSizePosition(width)
	{
		var s = _getDiaporamaSize();
		s.availableWidth += width ? width : 0;

		var margin_left = width ? (width < 0 ? width : 0) : -s.sidebarWidth;

		var css_loading =
		{
			top: s.barTopHeight,
			width: s.availableWidth,
			height: s.availableHeight
		};

		var css_inner = {width: s.availableWidth};

		var css_keyboard =
		{
			maxWidth: s.availableWidth,
			marginLeft: margin_left,
			height: s.availableHeight,
			top: s.barTopHeight
		};

		var css_text =
		{
			bottom: (s.barBottomHeight + s.carouselHeight),
			marginLeft: margin_left,
			maxWidth: s.availableWidth
		};

		if (width === undefined || !_options.animate)
		{
			$('#diaporama_inner').css(css_inner);
			$('#diaporama_keyboard').css(css_keyboard);
			$('#diaporama_loading').css(css_loading);
			$('#diaporama_text').css(css_text);
		}
		else
		{
			$('#diaporama_inner').animate(css_inner, _options.sidebarShowDuration, 'swing');
			$('#diaporama_keyboard').animate(css_keyboard, _options.sidebarShowDuration, 'swing');
			$('#diaporama_loading').animate(css_loading, _options.sidebarShowDuration, 'swing');
			$('#diaporama_text').animate(css_text, _options.sidebarShowDuration, 'swing');
		}
	}

	/**
	 * Change le fichier en fonction du bouton de navigation cliqué.
	 *
	 * @param object button
	 *   Bouton de navigation qui a été cliqué.
	 * @param boolean click
	 *   Indique si la fonction a été appelée par un click.
	 *
	 * @return void
	 */
	function _changeItem(button, click)
	{
		// On met sur pause toutes les vidéos.
		$('#diaporama_item video').each(function(){this.pause()});

		// On détermine la position du fichier à afficher.
		var new_current_position;
		switch (button)
		{
			case 'first' :
				new_current_position = 1;
				break;

			case 'prev' :
				new_current_position = _currentPosition - 1;
				break;

			case 'next' :
				new_current_position = _currentPosition + 1;
				break;

			case 'last' :
				new_current_position = _itemsCount;
				break;
		}

		// Si le fichier n'existe pas
		// ou correspond à une valeur impossible
		// on bien si un effet de transition entre fichiers est en cours,
		// on ne va pas plus loin.
		if (!$('#diaporama_item_' + new_current_position).length
		|| new_current_position < 1 || new_current_position > _itemsCount
		|| _itemTransitionLock || _sidebarMoveLock)
		{
			return;
		}

		// On redimensionne le fichier à afficher.
		_realsize = false;
		_zoom = {};
		$('#diaporama_navigation_switch').removeClass('fullsize');

		// On indique les bonnes dimensions et position au fichier que l'on va afficher.
		_changeItemSizePosition(new_current_position);

		// On modifie la valeur de la position courante.
		var old_current_position = _currentPosition;
		_currentPosition = new_current_position;

		// Transition entre le fichier courant et le fichier demandé.
		_itemTransition(old_current_position, new_current_position, button);

		// On effectue une nouvelle requête ajax, mais seulement
		// après l'effet de transition si un effet de transition est activé,
		// afin d'éviter que l'effet de transition ne soit saccadé.
		if (_options.transitionEffect == 'none')
		{
			_getData(false, false, _currentPosition, _currentPosition);
			_changeItemInfos();
		}
		else
		{
			setTimeout(function()
			{
				_getData(false, false, _currentPosition, _currentPosition);
				_changeItemInfos();
			},
			_options.transitionDuration);
		}

		// Change les boutons de navigation.
		_changeButtonsNavigation();

		// En lecture automatique, si l'on est arrivé au dernier
		// fichier et que l'option pour boucler est désactivée,
		// on stoppe la lecture automatique.
		if (_autoActive && new_current_position == _itemsCount && !_options.autoLoop)
		{
			_icon('stop').click();
		}

		// Si le changement de fichier s'est fait manuellement,
		// et que la lecture automatique est en cours,
		// alors on réinitialise la lecture automatique.
		if (click && typeof _autoTimer == 'number')
		{
			// On stoppe la lecture automatique.
			clearTimeout(_autoTimer);
			_autoTimer = undefined;

			// On redémarre la lecture automatique.
			_autoChangeItem(true);
		}
	}

	/**
	 * Change les informations du fichier courant
	 * affichées dans le diaporama.
	 *
	 * @return void
	 */
	function _changeItemInfos()
	{
		if (_items[_currentPosition] === undefined)
		{
			return;
		}

		var current = _items[_currentPosition];
		var html;

		// Identifiant du fichier.
		diaporama_item_id = current.id;

		// Fil d'Ariane.
		$('#diaporama_breadcrumb').html(current.breadcrumb);

		// Bouton de téléchargement.
		if (current.download && $('#small_screen').is(':hidden'))
		{
			_icon('download')
				.attr('title', diaporama_download_text
					+ ' (' + current.stats.filesize.text + ')')
				.attr('href', $('<input>').html(current.download).text())
				.show();
			$('#diaporama_item_' + _currentPosition).unbind('contextmenu');
		}
		else
		{
			_icon('download').hide();

			// Désactivation du menu contextuel.
			$('#diaporama_item_' + _currentPosition).bind('contextmenu', function()
			{
				return false;
			});
		}

		// Édition.
		if (current.user.edit && $('#small_screen').is(':hidden'))
		{
			_icon('edit').show();

			$('#diaporama_edit_title').val($('<input>').html(current.title).text());
			$('#diaporama_edit_filename').val($('<input>').html(current.filename).text());
			$('#diaporama_edit_desc').val($('<input>').html(current.description_edit).text());

			var tags = '';
			if (current.tags.length)
			{
				for (var tag in current.tags)
				{
					tags += ', ' + current.tags[tag].name;
				}
				tags = tags.substring(2);
			}
			$('#diaporama_edit_tags').val($('<input>').html(tags).text());
		}
		else
		{
			if (_icon('edit').hasClass('active'))
			{
				_icon('edit').click();
			}
			_icon('edit').hide();
		}

		// Informations : titre et description.
		if (current.description)
		{
			$('#diaporama_sidebar_description p').html(current.description);
			$('#diaporama_sidebar_description').show();
		}
		else
		{
			$('#diaporama_sidebar_description').hide();
		}
		_changeOverImageTextDisplay();

		// Informations : statistiques.
		for (var stat in current.stats)
		{
			if (Object.keys(current.stats[stat]).length)
			{
				if (stat == 'rating')
				{
					html = '';
					var entity;
					for (var key in current.stats[stat].array)
					{
						if (current.stats[stat].array[key] == 1)
						{
							entity = '&#xe9d9;';
						}
						else if (current.stats[stat].array[key] == 0)
						{
							entity = '&#xe9d7;';
						}
						else
						{
							entity = '&#xe9d8;';
						}
						html += '<span class="rating">' + entity + '</span>';
					}
					$('#diaporama_sidebar_stats_' + stat + ' span')
						.html(html).attr('title', current.stats[stat].formated);
				}
				else
				{
					$('#diaporama_sidebar_stats_' + stat + ' span')
						.text(current.stats[stat].short);
				}
				$('#diaporama_sidebar_stats_' + stat).show();
			}
			else
			{
				$('#diaporama_sidebar_stats_' + stat).hide();
			}
		}

		// Informations : utilisateur.
		if (current.owner)
		{
			$('#diaporama_sidebar_owner .diaporama_sidebar_content').html(current.owner);
		}
		else
		{
			$('#diaporama_sidebar_owner').hide();
		}

		// Informations : propriétés.
		$('#diaporama_sidebar_properties_type span').text(current.type_text);
		$('#diaporama_sidebar_properties_filesize span').text(current.stats.filesize.text);
		$('#diaporama_sidebar_properties_size span:first-child')
			.text(current.width + ' x ' + current.height);
		if (current.is_video)
		{
			$('#diaporama_sidebar_properties_duration').show();
			$('#diaporama_sidebar_properties_duration span').text(current.duration_text);
		}
		else
		{
			$('#diaporama_sidebar_properties_duration').hide();
		}
		if (current.date_created)
		{
			$('#diaporama_sidebar_properties_date_created').show();
			$('#diaporama_sidebar_properties_date_created span').html(current.date_created_text);
		}
		else
		{
			$('#diaporama_sidebar_properties_date_created').hide();
		}

		// Informations : tags.
		if (current.tags.length)
		{
			html = '<ul>';
			for (var tag in current.tags)
			{
				html += '<li><a href="' + current.tags[tag].link + '">'
					+ current.tags[tag].name + '</a></li>';
			}
			html += '</ul>';
			$('#diaporama_sidebar_tags .diaporama_sidebar_content').html(html);
			$('#diaporama_sidebar_tags').show();
		}
		else
		{
			$('#diaporama_sidebar_tags').hide();
		}

		// Informations : métadonnées.
		['exif', 'iptc', 'xmp'].forEach(function(meta)
		{
			if (Object.keys(current[meta]).length)
			{
				html = '<ul>';
				for (var data in current[meta])
				{
					html += '<li>' + current[meta][data].name + ' : <span>';
					if (current[meta][data].link)
					{
						html += '<a href="' + current[meta][data].link + '">'
							+ current[meta][data].value + '</a>';
					}
					else
					{
						html += current[meta][data].value;
					}
					html += '</span></li>';
				}
				html += '</ul>';
				$('#diaporama_sidebar_' + meta).show();
				$('#diaporama_sidebar_' + meta + ' .diaporama_sidebar_content').html(html);
			}
			else
			{
				$('#diaporama_sidebar_' + meta).hide();
			}
		});

		// Dans les favoris ?
		var fav = _icon('favorite');
		if (fav.length)
		{
			if (_items[_currentPosition].user.in_favorites)
			{
				fav.attr('title', diaporama_favorites_remove_text);
				fav.addClass('active');
				$('#diaporama_favorite').addClass('in');
			}
			else
			{
				fav.attr('title', diaporama_favorites_add_text);
				fav.removeClass('active');
				$('#diaporama_favorite').removeClass('in');
			}
		}

		// Dans la sélection ?
		var sel = _icon('selection');
		if (sel.length)
		{
			if (_items[_currentPosition].in_selection)
			{
				sel.attr('title', diaporama_selection_remove_text);
				sel.addClass('active');
				$('#diaporama_selection').addClass('in');
			}
			else
			{
				sel.attr('title', diaporama_selection_add_text);
				sel.removeClass('active');
				$('#diaporama_selection').removeClass('in');
			}
		}

		// Carrousel.
		_carouselChangeCurrent();

		// Change le bouton de switch.
		_changeButtonSwitch();

		// Position dans la série.
		_changePosition();
	}

	/**
	 * Change les dimensions et la position (coordonnées spatiales) du fichier
	 * en fonction de l'espace disponible dans la zone d'affichage.
	 *
	 * @param int position
	 *   Position du fichier dans la série (entre 1 et itemsCount).
	 *   A ne pas confondre avec la position (top, left) du fichier
	 *   dans la zone d'affichage.
	 * @param bool animate
	 *   Doit-on autoriser l'animation du fichier ?
	 * @param bool visible
	 *   Doit-on afficher le fichier ?
	 * @param bool return_css
	 *   Doit-on retourner les règles CSS au lieu de les appliquer au fichier ?
	 * @param int add_width
	 *   Largeur à ajouter à la largeur de la zone d'affichage.
	 *
	 * @return mixed
	 *   Retourne les valeurs CSS, si demandées.
	 */
	function _changeItemSizePosition(position, animate, visible, return_css, add_width)
	{
		if (_items[position] === undefined || !$('#diaporama_item_' + position).length)
		{
			return;
		}

		// Paramètres du diaporama.
		var s = _getDiaporamaSize();
		s.availableWidth += add_width ? add_width : 0;

		// Paramètres du fichier.
		var item = $('#diaporama_item_' + position);
		var item_width = _items[position].width_resized;
		var item_height = _items[position].height_resized;
		var item_contain_width = item_width;
		var item_contain_height = item_height;
		var item_width_resize = item_width;
		var item_height_resize = item_height;
		var width_ratio = item_width / s.availableWidth;
		var height_ratio = item_height / s.availableHeight;

		// Dimensions (redimensionnées) selon l'espace disponible.
		if ((item_width > s.availableWidth) && (width_ratio >= height_ratio))
		{
			item_contain_width = Math.round(s.availableWidth);
			item_contain_height = Math.round(item_height / width_ratio);
		}
		if ((item_height > s.availableHeight) && (height_ratio >= width_ratio))
		{
			item_contain_width = Math.round(item_width / height_ratio);
			item_contain_height = Math.round(s.availableHeight);
		}
		item.attr('data-contain-width', item_contain_width);
		item.attr('data-contain-height', item_contain_height);

		if (!_realsize)
		{
			item_height_resize = item_contain_height;
			item_width_resize = item_contain_width;
		}

		if (_zoom.width)
		{
			if (_zoom.width < item_contain_width)
			{
				item_height_resize = item_contain_height;
				item_width_resize = item_contain_width;
			}
			else
			{
				item_height_resize = _zoom.height;
				item_width_resize = _zoom.width;
			}
		}

		// Position du fichier.
		var item_offset = item.offset();
		var item_current_offset = item.offset();

		// Position du fichier, largeur : modification de la taille du fichier.
		if (item_width_resize <= s.availableWidth || item_width_resize != item.width())
		{
			item_offset.left = (s.availableWidth - item_width_resize) / 2;
		}

		// Position du fichier, largeur : modification de la taille de la zone d'affichage.
		else if (item_width_resize > s.availableWidth
		&& s.availableWidth != _previousDiaporamaSize.availableWidth)
		{
			item_offset.left += (s.availableWidth - _previousDiaporamaSize.availableWidth) / 2;

			var right = item_width_resize - s.availableWidth + item_offset.left;
			if (right < 0)
			{
				item_offset.left -= right;
			}
			if (item_offset.left > 0)
			{
				item_offset.left = 0;
			}
		}

		// Position du fichier, hauteur : modification de la taille du fichier.
		if (item_height_resize <= s.availableHeight || item_height_resize != item.height())
		{
			item_offset.top = ((s.availableHeight - item_height_resize) / 2) + s.barTopHeight;
		}

		// Position du fichier, hauteur : modification de la taille de la zone d'affichage.
		else if (item_height_resize > s.availableHeight
		&& s.availableHeight != _previousDiaporamaSize.availableHeight)
		{
			item_offset.top += ((s.availableHeight - _previousDiaporamaSize.availableHeight) / 2);
			var bottom = item_height_resize
				- (s.availableHeight + s.barBottomHeight + s.carouselHeight)
				+ item_offset.top;
			if (bottom < 0)
			{
				item_offset.top -= bottom;
			}
			if (item_offset.top > s.barTopHeight)
			{
				item_offset.top = s.barTopHeight;
			}
		}

		// Valeurs CSS.
		var new_width = Math.round(item_width_resize);
		var new_height = Math.round(item_height_resize);
		var css =
		{
			top: Math.round(item_offset.top),
			left: Math.round(item_offset.left),
			width: new_width,
			height: new_height
		};

		// On applique les nouvelles propriétés au fichier.
		if (!return_css)
		{
			if (animate && item.is(':visible')
			&& _options.itemResizeDuration > 0
			&& _options.animate)
			{
				item.animate(css, _options.itemResizeDuration);
			}
			else
			{
				item.css(css);
			}
			$('#diaporama_favorite,#diaporama_selection').css({left: s.availableWidth});
		}

		// Doit-on afficher le fichier ?
		if (visible)
		{
			item.show();
		}

		// Doit-on retourner les valeurs CSS ?
		if (return_css)
		{
			return css;
		}

		// Pour le fichier courant.
		if (position == _currentPosition)
		{
			_changeButtonSwitch(new_width, new_height);
			_changePosition(new_width);
		}
	}

	/**
	 * Change le contenu et l'affichage du titre
	 * et de la description par dessus les images.
	 *
	 * @return void
	 */
	function _changeOverImageTextDisplay()
	{
		var current = _items[_currentPosition];

		$('#diaporama_title').html(current.title).scrollTop();
		$('#diaporama_description').html(current.description).scrollTop();

		// Affichage de la description.
		if (_options.overImageDescription && current.description)
		{
			$('#diaporama_description').show();
		}
		else
		{
			$('#diaporama_description').hide();
		}

		// Affichage du titre.
		if (_options.overImageTitle)
		{
			$('#diaporama_title').show();
		}
		else
		{
			$('#diaporama_title').hide();
		}

		// Affichage de la zone de texte.
		if (current.is_video || !(_options.overImageDescription || _options.overImageTitle))
		{
			$('#diaporama_text').hide();
		}
		else
		{
			$('#diaporama_text').show();
		}
	}

	/**
	 * Change le texte pour la position dans la série.
	 *
	 * @param int width
	 *   Largeur du fichier.
	 *
	 * @return void
	 */
	function _changePosition(width)
	{
		if (_items[_currentPosition] !== undefined)
		{
			var text = $(window).width() <= _options.mobileMaxWidth
				? _items[_currentPosition].position
				: _items[_currentPosition].position_text;
			if ($(window).width() > (_options.mobileMaxWidth + 100))
			{
				text += ' - ';
				text += Math.round(
					((width || $('#diaporama_item_' + _currentPosition).width())
						/ _items[_currentPosition].width_resized) * 100
				);
				text += '%';
			}
			$('#diaporama_position').text(text);
		}
	}

	/**
	 * Création de l'élément "<img>" ou "<video>" du fichier correspondant à "position".
	 *
	 * @param string position
	 *   Position du fichier dans la série.
	 *
	 * @return void
	 */
	function _createItem(position)
	{
		if (_items[position] === undefined
		|| $('#diaporama_item_' + position).length)
		{
			return;
		}

		if (_items[position].is_video)
		{
			var loop = diaporama_video_loop ? ' loop' : '';
			var muted = diaporama_video_muted ? ' muted' : '';
			var html =
				'<video' + loop + muted
				+ ' style="display:none" class="diaporama_item" id="diaporama_item_'
				+ position + '" controls poster="'
				+ _items[position].poster + '"><source src="' + _items[position].source
				+ '" type="' + _items[position].type_mime + '"></video>';
			$('#diaporama_item').append(html);
			$('#diaporama_item video').prop('volume', 0.5);
		}
		else
		{
			$('#diaporama_item').append(
				'<img data-type="' + _items[position].type_mime + '"'
				+ ' class="diaporama_item" id="diaporama_item_' + position
				+ '" style="display:none" src="' + _items[position].source + '">'
			);
		}

		// Icône de chargement.
		if (position == _currentPosition)
		{
			$('#diaporama_loading').hide();
		}
	}

	/**
	 * Supprime les fichiers inutiles pour ne pas encombrer et ralentir le diaporama.
	 *
	 * @param bool all
	 *   Doit-on supprimer touts les fichiers ?
	 *
	 * @return void
	 */
	function _deleteItems(all)
	{
		$('.diaporama_item').each(function()
		{
			var position = $(this).attr('id').replace(/diaporama_item_/, '');
			if (_items[position] === undefined || all)
			{
				$(this).remove();
			}
		});
	}

	/**
	 * Gestion du déplacement de l'image.
	 *
	 * @param object img
	 * @param object evt
	 *
	 * @return bool
	 */
	function _dragImage(img, evt)
	{
		var s = _getDiaporamaSize();
		var x = img.offsetLeft;
		var y = img.offsetTop;
		var mx = evt.pageX;
		var my = evt.pageY;

		$(document).mousemove(function(e)
		{
			var newmx = e.pageX;
			var newmy = e.pageY;
			x += newmx - mx;
			y += newmy - my;
			mx = newmx;
			my = newmy;

			if (img.width > s.availableWidth)
			{
				if (x > 0)
				{
					x = 0;
				}
				else if (x < (s.availableWidth - img.width))
				{
					x = s.availableWidth - img.width;
				}
			}
			else
			{
				x = img.offsetLeft;
			}

			if (img.height > s.availableHeight)
			{
				if (y > s.barTopHeight)
				{
					y = s.barTopHeight;
				}
				else if (y < (s.barTopHeight + s.availableHeight - img.height))
				{
					y = s.barTopHeight + s.availableHeight - img.height;
				}
			}
			else
			{
				y = img.offsetTop;
			}

			img.style.left = x + 'px';
			img.style.top = y + 'px';

			return false;
		});

		$(document).mouseup(function()
		{
			$(document).unbind('mousemove mouseup');

			return false;
		});

		return false;
	}

	/**
	 * Gestion du déplacement de l'image sur mobile.
	 *
	 * @param object evt
	 *
	 * @return void
	 */
	function _dragImageTouch(evt)
	{
		var img = document.getElementById('diaporama_item_' + _currentPosition);
		var s = _getDiaporamaSize();
		var x = img.offsetLeft;
		var y = img.offsetTop;
		var mx = evt.changedTouches[0].pageX;
		var my = evt.changedTouches[0].pageY;

		var touchmove = function(e)
		{
			var touchobj = e.changedTouches[0];

			var newmx = touchobj.pageX;
			var newmy = touchobj.pageY;
			x += newmx - mx;
			y += newmy - my;
			mx = newmx;
			my = newmy;

			if (img.width > s.availableWidth)
			{
				if (x > 0)
				{
					x = 0;
				}
				else if (x < (s.availableWidth - img.width))
				{
					x = s.availableWidth - img.width;
				}
			}
			else
			{
				x = parseInt(img.style.left);
			}

			if (img.height > s.availableHeight)
			{
				if (y > s.barTopHeight)
				{
					y = s.barTopHeight;
				}
				else if (y < (s.barTopHeight + s.availableHeight - img.height))
				{
					y = s.barTopHeight + s.availableHeight - img.height;
				}
			}
			else
			{
				y = img.offsetTop;
			}

			img.style.left = x + 'px';
			img.style.top = y + 'px';
		};

		var touchend = function()
		{
			img.removeEventListener('touchmove', touchmove);
			img.removeEventListener('touchend', touchend);
		};

		img.addEventListener('touchmove', touchmove);
		img.addEventListener('touchend', touchend);
	}

	/**
	 * Affichage en mode plein écran.
	 *
	 * @return void
	 */
	function _fullScreen()
	{
		if (_options.fullScreen)
		{
			var el = document.body;
			var fs = el.requestFullScreen || el.webkitRequestFullScreen
				 || el.mozRequestFullScreen || el.msRequestFullScreen;
			if (fs)
			{
				fs.call(el);
			}
		}
	}

	/**
	 * Sortie du mode plein écran.
	 *
	 * @return void
	 */
	function _fullScreenExit()
	{
		if (document.fullScreen || document.mozFullScreen
		|| document.webkitIsFullScreen || document.msFullscreenElement)
		{
			if (document.exitFullscreen)
			{
				document.exitFullscreen();
			}
			else if (document.mozCancelFullScreen)
			{
				document.mozCancelFullScreen();
			}
			else if (document.webkitExitFullscreen)
			{
				document.webkitExitFullscreen();
			}
			else if (document.msExitFullscreen)
			{
				document.msExitFullscreen();
			}
		}
	}

	/**
	 * Récupération des données depuis le serveur.
	 *
	 * @param bool start
	 *   Utilise-t-on cete fonction pour la première fois ?
	 * @param bool only_item_infos
	 *   Ne changer que les informations du fichier courant ?
	 * @param int item_position
	 *   Position du fichier courant dans la série.
	 *   0 pour ne pas récupérer d'informations du fichier.
	 * @param int carousel_position
	 *   Position dans le carrousel.
	 *   0 pour ne pas récupérer d'informations du carrousel.
	 *
	 * @return void
	 */
	function _getData(start, only_item_infos, item_position, carousel_position)
	{
		if (start)
		{
			$('#diaporama_loading').show();
		}

		_carouselCurrentPosition = carousel_position > 0
			? carousel_position
			: item_position;

		$.post(
			gallery_path + '/ajax.php',
			{
				anticsrf: anticsrf,
				section: 'diaporama',
				q: q,
				get_item: item_position > 0 ? 1 : 0,
				get_carousel: carousel_position > 0 ? 1 : 0,
				key: diaporama_key,
				thumb_size: diaporama_thumb_size,
				item_position: item_position,
				item_limit: _options.itemLimit,
				carousel_position: _carouselCurrentPosition
			},
			function(r)
			{
				if (typeof r != 'object' || r === null)
				{
					return;
				}
				switch (r.status)
				{
					// Erreur.
					case 'error' :
						$('.diaporama_sidebar[data-name="informations"] > div').hide();
						console.log(r.message);
						_alertError(r.message);
						break;

					// Aucun résultat.
					case 'no_result' :
						break;

					// Succès.
					case 'success' :
						$('.diaporama_sidebar[data-name="informations"] > div').show();

						// Gestion du carrousel.
						var carousel = function()
						{
							// Informations des vignettes du carrousel.
							_carouselThumbs = r.carousel;

							// Si le nombre de fichiers de la section courante
							// n'est plus le même, on recharge le diaporama.
							if (_itemsCount > 0 && r.count != _itemsCount)
							{
								_reload(_currentPosition);
							}

							// Si des fichiers ont été supprimés, on recharge le diaporama.
							if (_carouselThumbs[carousel_position] === undefined && r.count > 0)
							{
								_reload(parseInt(r.count));
								return;
							}

							_itemsCount = r.count;
							_carouselChangeThumbs();
						};
						if (carousel_position)
						{
							carousel();
						}

						// Si on ne doit pas récupérer les informations du fichier courant,
						// on ne va pas plus loin.
						if (!item_position)
						{
							break;
						}

						// Change uniquement les informations
						// du fichier courant ?
						if (only_item_infos)
						{
							_items = r.items;
							_changeItemInfos();
							break;
						}

						// Si la série ne contient plus aucun fichier,
						// on vide l'interface du diaporama.
						if (r.count == 0)
						{
							$('#diaporama_item,#diaporama_breadcrumb,#diaporama_position,'
								+ '.diaporama_sidebar_inner,#diaporama_carousel_thumbs,'
								+ '#diaporama_auto_message,#diaporama_auto_mode').empty();
							$('.diaporama_icon[data-name="download"],'
								+ '.diaporama_icon[data-name="favorite"]')
								.unbind().removeClass('active').removeAttr('title');
							$('a[data-name="carousel_prev"],a[data-name="carousel_next"]')
								.unbind().addClass('disabled');
							$('#diaporama_navigation > a').unbind().addClass('disabled');
							$('#diaporama_auto_mode > a').unbind();
							$('#diaporama_switch').unbind().removeClass().addClass('disabled');
							_carouselThumbs = undefined;
							_currentPosition = 0;
						}

						// Si la série ou le fichier courant ont été modifié,
						// on recrée tous les fichiers.
						var remake = false;
						for (var pos in r.items)
						{
							if (_items[pos] === undefined
							 || _items[pos].md5 == r.items[pos].md5)
							{
								continue;
							}

							if (_currentPosition > r.count)
							{
								_currentPosition = r.count;
							}

							_deleteItems(true);
							remake = true;
							break;
						}

						_items = r.items;
						_itemsCount = r.count;

						// Création des fichiers.
						var preload = [1, _currentPosition, _itemsCount];
						var preload_push = function(p)
						{
							if (p > 1 && p < _itemsCount && !preload.includes(p))
							{
								preload.push(p);
							}
						};
						for (var i = 1; i <= _options.itemPreload; i++)
						{
							preload_push(i + 1);
							preload_push(_itemsCount - i);
							preload_push(_currentPosition - i);
							preload_push(_currentPosition + i);
						}
						for (var pos in r.items)
						{
							if (!preload.includes(parseInt(pos)))
							{
								continue;
							}

							var current = _currentPosition == pos;

							if ((!start && !remake) && current)
							{
								continue;
							}

							_createItem(pos);
							_changeItemSizePosition(pos, false, current);

							if ((start || remake) && current)
							{
								_changeItemInfos();
							}
						}

						// Boutons de navigation.
						_changeButtonsNavigation();

						// On indique de regénérer le carrousel.
						if (remake)
						{
							$('#diaporama_carousel_thumbs').empty();
						}

						// Suppression des fichiers inutiles.
						_deleteItems();

						// Démarrage de la lecture automatique au lancement ?
						if (start && _options.autoStart)
						{
							_icon('start').click();
						}

						// On met à jour les informations (notamment pour le nombre de vues).
						_changeItemInfos();

						// Changement des vignettes du carrousel.
						_carouselMaxPosition = 0;
						_carouselChangeThumbs();
						break;

					default :
						console.log(r);
				}
			},
			'json'
		);
	}

	/**
	 * Retourne les dimensions des éléments du diaporama.
	 *
	 * @return object
	 */
	function _getDiaporamaSize()
	{
		var s = {barBottomHeight: 0, barTopHeight: 0, carouselHeight: 0, sidebarWidth: 0};

		// Largeur du panneau latéral.
		if (_options.sidebarItemResize && $('.diaporama_sidebar').is(':visible'))
		{
			s.sidebarWidth = $('.diaporama_sidebar').outerWidth();
		}

		// Hauteur du carrousel.
		if (_options.carousel && _options.controlBars)
		{
			s.carouselHeight = $('#diaporama_carousel').outerHeight();
		}

		// Hauteur des barres du haut et du bas.
		if (_options.controlBars)
		{
			s.barTopHeight = $('#diaporama_top').outerHeight();
			s.barBottomHeight = $('#diaporama_bottom').outerHeight();
		}

		// Dimensions de la zone d'affichage.
		s.availableHeight = $(window).height()
			- s.barTopHeight - s.barBottomHeight - s.carouselHeight;
		s.availableWidth = $(window).width() - s.sidebarWidth;

		return s;
	}

	/**
	 * Retourne l'élément d'une icône.
	 *
	 * @return object
	 */
	function _icon(name)
	{
		return $('.diaporama_icon[data-name="' + name + '"]');
	}

	/**
	 * Détermine si l'on est sur un mobile.
	 *
	 * @return bool
	 */
	function _isMobile()
	{
		var touchscreen = !!('ontouchstart' in window)
			|| !!('ontouchstart' in document.documentElement)
			|| !!window.ontouchstart || !!window.Touch || !!window.onmsgesturechange
			|| (window.DocumentTouch && window.document instanceof window.DocumentTouch);

		return touchscreen && $(window).width() <= _options.mobileMaxWidth;
	}

	/**
	 * Effectue la transition entre le fichier courant et le fichier demandé.
	 *
	 * @param int old_position
	 * @param int new_position
	 * @param string button
	 *
	 * @return void
	 */
	function _itemTransition(old_position, new_position, button)
	{
		var item_old = $('#diaporama_item_' + old_position);
		var item_new = $('#diaporama_item_' + new_position);
		var old_offset = item_old.offset();
		var duration = parseInt(_options.transitionDuration);
		var s = _getDiaporamaSize();
		var transition = _options.transitionEffect;

		_itemTransitionLock = true;

		if (_options.transitionEffect == 'random')
		{
			var nb = $('#diaporama_transitions_effect option').length - 2;
			var rand = Math.floor(Math.random() * nb) + 1;
			transition = document.getElementById('diaporama_transitions_effect')
				.getElementsByTagName('option')[rand].value;
		}

		switch (transition)
		{
			case 'fade' :
				item_old.fadeOut(duration);
				item_new.fadeIn(duration);
				break;

			case 'slideX' :
			case 'slideY' :
				var old_css_animate = {};
				var old_css = { display: 'none' };
				if (transition == 'slideX')
				{
					old_css_animate.left = button == 'next' || button == 'last'
						? old_offset.left - s.availableWidth
						: old_offset.left + s.availableWidth;
					old_css.left = old_offset.left;
				}
				else
				{
					old_css_animate.top = button == 'next' || button == 'last'
						? old_offset.top + s.availableHeight
						: old_offset.top - s.availableHeight;
					old_css.top = old_offset.top;
				}
				item_old.animate(old_css_animate, duration, 'swing', () => item_old.css(old_css));

				var new_css = { display: 'block' };
				var new_css_animate = {};
				if (transition == 'slideX')
				{
					var img_new_left = item_new.css('left');
					new_css_animate.left = img_new_left;
					new_css.left = button == 'next' || button == 'last'
						? parseFloat(img_new_left) + s.availableWidth
						: parseFloat(img_new_left) - s.availableWidth;
				}
				else
				{
					var img_new_top = item_new.css('top');
					new_css_animate.top = img_new_top;
					new_css.top = button == 'next' || button == 'last'
						? parseFloat(img_new_top) - s.availableHeight
						: parseFloat(img_new_top) + s.availableHeight;
				}
				item_new.css(new_css).animate(new_css_animate, duration, 'swing');
				break;

			case 'slideXLeft' :
			case 'slideYBottom' :
				var old_css_animate = {};
				var old_css = { display: 'none' };
				if (transition == 'slideXLeft')
				{
					old_css_animate.left = old_offset.left - s.availableWidth;
					old_css.left = old_offset.left;
				}
				else
				{
					old_css_animate.top = old_offset.top + s.availableHeight;
					old_css.top = old_offset.top;
				}
				item_old.animate(old_css_animate, duration / 2, 'easeOutQuad', function()
				{
					item_old.css(old_css);

					var new_css = { display : 'block' };
					var new_css_animate = {};
					if (transition == 'slideXLeft')
					{
						var img_new_left = item_new.css('left');
						new_css_animate.left = img_new_left;
						new_css.left = -(parseFloat(img_new_left) + s.availableWidth);
					}
					else
					{
						var img_new_top = item_new.css('top');
						new_css_animate.top = img_new_top;
						new_css.top = parseFloat(img_new_top) + s.availableHeight;
					}
					item_new
						.css(new_css)
						.animate(new_css_animate, duration / 2, 'easeOutQuad');
				});
				break;

			case 'zoom' :
			case 'curtainX' :
			case 'curtainY' :
				var old_css_animate = {};
				var old_css = { display: 'none' };
				if (transition != 'curtainX')
				{
					old_css_animate.height = 0;
					old_css_animate.top = s.availableHeight / 2;
					old_css.top = old_offset.top;
				}
				if (transition != 'curtainY')
				{
					old_css_animate.width = 0;
					old_css_animate.left = s.availableWidth / 2;
					old_css.left = old_offset.left;
				}
				item_old.animate(old_css_animate, duration / 2, 'swing', function()
				{
					item_old.css(old_css);

					var new_css = { display: 'block' };
					var new_css_animate = {};
					if (transition != 'curtainX')
					{
						new_css_animate.height = item_new.height();
						new_css_animate.top = item_new.css('top');
						new_css.top = s.availableHeight / 2;
						new_css.height = 0;
					}
					if (transition != 'curtainY')
					{
						new_css_animate.width = item_new.width();
						new_css_animate.left = item_new.css('left');
						new_css.left = s.availableWidth / 2;
						new_css.width = 0;
					}
					item_new.css(new_css).animate(new_css_animate, duration / 2, 'swing');
				});
				break;

			case 'puff' :
				var old_css_animate =
				{
					height: item_old.height() * 1.5,
					width: item_old.width() * 1.5,
					left: (s.availableWidth - (item_old.width() * 1.5)) / 2,
					top: ((s.availableHeight - (item_old.height() * 1.5)) / 2) + s.barTopHeight,
					opacity: 0
				};
				var old_css = { display: 'none', opacity: 1 };
				item_old.animate(old_css_animate, duration / 2, 'swing', function()
				{
					item_old.css(old_css);

					var new_css = { display: 'block' };
					new_css.height = item_new.height() * 1.5;
					new_css.width = item_new.width() * 1.5;
					new_css.left = (s.availableWidth - new_css.width) / 2;
					new_css.top = ((s.availableHeight - new_css.height) / 2) + s.barTopHeight;
					new_css.opacity = 0;

					var new_css_animate =
						_changeItemSizePosition(new_position, true, true, true);
					new_css_animate.opacity = 1;

					item_new.css(new_css).animate(new_css_animate, duration / 2, 'swing');
				});
				break;

			default :
				item_old.hide();
				item_new.show();
				duration = 0;
				break;
		}

		setTimeout(function() { _changeItemInfos(); _itemTransitionLock = false; }, duration);
	}

	/**
	 * Gestion du contrôle au clavier.
	 *
	 * @return void
	 */
	function _keyboard()
	{
		if (!_options.keyboard)
		{
			$('#diaporama_keyboard_k').hide();
			return;
		}

		var ctrl = false;
		var ctrl_timer;

		// On désactive le contrôle au clavier lorsque
		// le focus est sur un élément de formulaire.
		var selector = '#diaporama select, #diaporama input, #diaporama textarea';
		$(selector).focus(() => _keyboardActive = false).blur(() => _keyboardActive = true);
		$('#diaporama_inner').click(() => $(selector).blur());

		$(document).keyup(function(e)
		{
			if (!_keyboardActive)
			{
				return;
			}
			if (e.keyCode == 16 || e.keyCode == 17 || e.keyCode == 18 || ctrl)
			{
				clearTimeout(ctrl_timer);
				ctrl_timer = setTimeout(() => ctrl = false, 1000);
				return;
			}

			switch (e.keyCode)
			{
				// Echap : quitte le diaporama.
				case 27 :
					_icon('close').click();
					break;

				// Espace : démarre ou arrête la lecture automatique.
				case 32 :
					if (_autoActive)
					{
						_icon('stop').click();
					}
					else
					{
						_icon('start').click();
					}
					break;

				// Haut de page : page de vignette précédente.
				case 33 :
					_icon('carousel_prev').click();
					break;

				// Bas de page : page de vignette suivante.
				case 34 :
					_icon('carousel_next').click();
					break;

				// Fin : dernier fichier.
				case 35 :
					_icon('last').click();
					break;

				// Début : premier fichier.
				case 36 :
					_icon('first').click();
					break;

				// Flèche gauche : fichier précédent.
				case 37 :
					_icon('prev').click();
					break;

				// Flèche droite : fichier suivant.
				case 39 :
					_icon('next').click();
					break;

				// C : Carrousel.
				case 67 :
					$('#diaporama_carousel_option').click();
					break;

				// D : Description sur les images.
				case 68 :
					$('#diaporama_over_image_description').click();
					break;

				// F : Ajoute le fichier aux favoris.
				case 70 :
					_icon('favorite').click();
					break;

				// I : Panneau d'informations.
				case 73 :
					if ($('#diaporama_top').is(':visible') && !_controlBarsAnimation)
					{
						_icon('informations').click();
					}
					break;

				// K : Affiche la liste des touches pour le contrôle au clavier.
				case 75 :
					$('#diaporama_keyboard').css('display',
						$('#diaporama_keyboard').is(':hidden') ? 'flex' : 'none');
					break;

				// S : Ajoute le fichier à la sélection.
				case 83 :
					_icon('selection').click();
					break;

				// T : Titre sur les images.
				case 84 :
					$('#diaporama_over_image_title').click();
					break;

				// Sauvegarde la nouvelle durée d'affichage.
				case 107 :
				case 109 :
					_savePrefs();
					break;

				// Point (pavé numérique) : switch taille réelle / redimensionnée.
				case 110 :
					$('#diaporama_switch').click();
					break;
			}
		});

		$(document).keydown(function(e)
		{
			if (!_keyboardActive || ctrl)
			{
				return;
			}

			switch (e.keyCode)
			{
				// Si les touches Alt, Ctrl ou Maj sont enfoncées,
				// on désactive le contrôle au clavier.
				case 16 :
				case 17 :
				case 18 :
					clearTimeout(ctrl_timer);
					ctrl = true;
					break;

				// Signe plus du pavé numérique :
				// augmente la durée d'affichage de la lecture automatique.
				case 107 :
					_autoDurationMouseDown = true;
					_autoChangeDuration(_options.autoDurationPrecision);
					_autoDurationMouseDown = false;
					break;

				// Signe moins du pavé numérique :
				// diminue la durée d'affichage de la lecture automatique.
				case 109 :
					_autoDurationMouseDown = true;
					_autoChangeDuration(-_options.autoDurationPrecision);
					_autoDurationMouseDown = false;
					break;
			}
		});

		// Gestion du focus sur les vidéos.
		$('body').on('focus', '#diaporama_item video', () => _keyboardActive = false);
		$('body').on('blur', '#diaporama_item video', () => _keyboardActive = true);
	}

	/**
	 * Recharge tout le diaporama depuis la position p.
	 *
	 * @param int p
	 *
	 * @return void
	 */
	function _reload(p)
	{
		_currentPosition = p;
		_itemsCount = 0;
		_realsize = false;
		_changeButtonsNavigation();
		_deleteItems(true);
		_getData(true, false, p, p);
	}

	/**
	 * Redimensionne le diaporama et le fichier courant
	 * lors du redimensionnement du navigateur.
	 *
	 * @return void
	 */
	function _resize()
	{
		_carouselChangeSizePosition();
		_changeCenterSizePosition();

		if (_items[_currentPosition] !== undefined)
		{
			_changeItemSizePosition(_currentPosition);

			if (_items[_currentPosition].user.edit && $('#small_screen').is(':hidden'))
			{
				_icon('edit').show();
			}
			else
			{
				if (_icon('edit').hasClass('active'))
				{
					_icon('edit').click();
				}
				_icon('edit').hide();
			}
			if (_items[_currentPosition].download && $('#small_screen').is(':hidden'))
			{
				_icon('download').show();
			}
			else
			{
				_icon('download').hide();
			}
		}

		_previousDiaporamaSize = _getDiaporamaSize();
	}

	/**
	 * Sauvegarde les préférences utilisateur.
	 *
	 * @return void
	 */
	function _savePrefs()
	{
		var prefs =
		{
			autoDuration: _options.autoDuration,
			autoLoop: _options.autoLoop,
			autoStart: _options.autoStart,
			carousel: _options.carousel,
			controlBars: _options.controlBars,
			fullScreen: _options.fullScreen,
			overImageDescription: _options.overImageDescription,
			overImageTitle: _options.overImageTitle,
			sidebarInformations: JSON.stringify(_options.sidebarInformations),
			showInformations: _options.showInformations,
			transitionDuration: _options.transitionDuration,
			transitionEffect: _options.transitionEffect,
		};
		document.cookie = 'igal_diaporama=' + JSON.stringify(prefs)
			+ '; expires=Fri, 31 Dec 9999 23:59:59 GMT; path=' + gallery_path;
	}

	/**
	 * Gestionnaires d'événements pour la lecture automatique.
	 *
	 * @return void
	 */
	function _setAutoEvents()
	{
		_icon('stop').hide();

		// Lecture.
		_icon('start').click(function()
		{
			// Si la lecture automatique a déjà été démarrée, inutile d'aller plus loin.
			if (_autoActive)
			{
				return;
			}

			// S'il n'y a pas plus d'un fichier, inutile de démarrer la lecture automatique.
			if (_itemsCount < 2)
			{
				return;
			}

			// Si l'on est au dernier fichier,
			// et que l'option pour boucler est désactivée,
			// on ne démarre pas la lecture automatique.
			if (_currentPosition == _itemsCount && !_options.autoLoop)
			{
				return;
			}

			// Changement des icônes.
			_icon('stop').show();
			_icon('start').hide();

			// Message indiquant le démarrage de la lecture automatique.
			_autoChangeMessage('PLAY');

			// On démarre la lecture automatique.
			_autoActive = true;
			_autoChangeItem(true);
		});

		// Arrêt.
		_icon('stop').click(function()
		{
			// Si la lecture automatique n'a pas été démarrée, inutile d'aller plus loin.
			if (!_autoActive)
			{
				return;
			}
			_autoActive = false;

			// On stoppe la lecture automatique.
			clearTimeout(_autoTimer);
			_autoTimer = undefined;

			// Message indiquant l'arrêt de la lecture automatique.
			_autoChangeMessage('STOP');

			// Changement des icônes.
			_icon('stop').hide();
			_icon('start').show();
		});

		// Augmenter la durée d'affichage.
		_icon('plus')
			.mousedown(function()
			{
				_autoDurationMouseDown = true;
				_autoChangeDuration(_options.autoDurationPrecision);
			})
			.mouseup(function() { _autoDurationMouseDown = false; _savePrefs(); });

		// Diminuer la durée d'affichage.
		_icon('minus')
			.mousedown(function()
			{
				_autoDurationMouseDown = true;
				_autoChangeDuration(-_options.autoDurationPrecision);
			})
			.mouseup(function() { _autoDurationMouseDown = false; _savePrefs(); });

		// Réglage de la durée d'affichage sur mobiles.
		$('body').on('input', '#diaporama_auto_duration_option input', function()
		{
			_options.autoDuration = parseFloat($(this).val());
			_autoTextDuration();
			_savePrefs();
		});
		
	}

	/**
	 * Définit les événements sur les boutons.
	 *
	 * @return void
	 */
	function _setButtonsEvents()
	{
		// Fermeture du diaporama.
		_icon('close').click(function()
		{
			// On réinitialise le zoom.
			_zoom = {};

			// On désactive les événements clavier.
			_keyboardActive = false;

			// On remet en place les barres de défilement.
			$('html').css('overflow', _overflow.html);
			$('body').css('overflow', _overflow.body);

			// On quitte le mode plein écran.
			_fullScreenExit();

			// On stoppe la lecture automatique.
			if (_autoActive)
			{
				_icon('stop').click();
				_autoChangeMessage('STOP');
			}

			// On stoppe la lecture vidéo.
			$('#diaporama_item video').each(function(){this.pause()});

			// On cache le diaporama et on réaffiche la galerie.
			$('#gallery').show();
			$('#diaporama').css('display', 'none');
			window.scroll(0, _overflow.scroll);
		});

		// Boutons de navigation.
		$('#diaporama_navigation a').each(function()
		{
			$(this).click(function()
			{
				if (!$(this).hasClass('disabled'))
				{
					_changeItem($(this).attr('data-name'), true);
				}
			});
		});

		// Bouton de switch taille réelle / taille redimensionnée.
		$('#diaporama_switch').click(function()
		{
			if (!$(this).hasClass('disabled'))
			{
				_realsize = !_realsize;

				if (_zoom.width)
				{
					_realsize = false;
					_zoom = {};
				}

				_changeItemSizePosition(_currentPosition, true);
			}
		});
		$('#diaporama_inner').dblclick(() => $('#diaporama_switch').click());
	}

	/**
	 * Événements pour le carrousel.
	 *
	 * @return void
	 */
	function _setCarouselEvents()
	{
		function change(d, p, n)
		{
			var id = $('#diaporama_carousel_thumbs dl:' + p).attr('id');

			if ($('a[data-name="carousel_' + d + '"]').hasClass('disabled')
			|| _carouselAnimation || $('#diaporama_carousel').is(':hidden') || !id)
			{
				return;
			}

			_carouselMaxPosition = 0;
			_carouselCurrentPosition = parseInt(id.replace(/diaporama_carousel_image_/, '')) - n;
			_carouselChangeThumbs(d);
		}

		$('a[data-name="carousel_prev"]').click(() => change('prev', 'first', 1));

		$('a[data-name="carousel_next"]').click(() =>
			change('next', 'eq(' + (Math.floor(_carouselMaxThumbs) - 1) + ')', 0));
	}

	/**
	 * Gestion de l'affichage des barres de contrôle.
	 *
	 * @return void
	 */
	function _setControlBarsEvents()
	{
		var hide_timer;

		// Pour éviter les caprices de certains navigateurs.
		var x, y;

		// Détermine si un panneau latéral est affiché.
		var is_visible = function()
		{
			var visible = false;

			$('.diaporama_sidebar').each(function()
			{
				if ($(this).is(':visible'))
				{
					visible = true;
					return;
				}
			});

			return visible;
		}

		if (!_options.controlBars)
		{
			$('.diaporama_control_bar,#diaporama_carousel').hide();
			_changeCenterSizePosition();
		}

		$('#diaporama').mousemove(function(e)
		{
			if (_options.controlBars || is_visible() || _controlBarsOver
			|| _controlBarsAnimation || (x == e.pageX && y == e.pageY))
			{
				return;
			}

			clearTimeout(hide_timer);

			x = e.pageX;
			y = e.pageY;

			// Affiche les barres.
			_controlBarsAnimation = true;
			setTimeout(() => _controlBarsAnimation = false, 350);

			$('.diaporama_control_bar' + (_options.carousel ? ',#diaporama_carousel' : ''))
				.fadeIn(350);
			if (_options.carousel)
			{
				_carouselChangeSizePosition();
				_changeCenterSizePosition();
			}

			// Cache les barres.
			hide_timer = setTimeout(function()
			{
				if (!_options.controlBars && !is_visible() && !_controlBarsOver)
				{
					_controlBarsAnimation = true;
					setTimeout(() => _controlBarsAnimation = false, 750);
					$('.diaporama_control_bar,#diaporama_carousel').fadeOut(750);
				}
			}, 1000);
		});

		// On ne cache pas les barres de contrôle
		// si la souris se trouve sur celles-ci.
		$('.diaporama_control_bar,#diaporama_carousel')
			.mouseover(() => _controlBarsOver = true)
			.mouseout(() => _controlBarsOver = false);
	}

	/**
	 * Gestion des favoris.
	 *
	 * @return void
	 */
	function _setFavoritesEvents()
	{
		_icon('favorite').click(function()
		{
			if (_items[_currentPosition] === undefined)
			{
				return;
			}

			var action = $(this).hasClass('active') ? 'remove' : 'add';

			if ((action == 'add' && _items[_currentPosition].user.in_favorites)
			|| ((action == 'remove' && !_items[_currentPosition].user.in_favorites)))
			{
				return;
			}

			if (action == 'remove')
			{
				$(this).removeClass('active');
				$('#diaporama_favorite').removeClass('in');
			}
			else
			{
				$(this).addClass('active');
				$('#diaporama_favorite').addClass('in');
			}

			$.post(gallery_path + '/ajax.php',
			{
				anticsrf: anticsrf,
				section: 'favorites-' + action,
				item_id: _items[_currentPosition].id
			},
			function(r)
			{
				if (typeof r != 'object' || r === null)
				{
					return;
				}
				switch (r.status)
				{
					// Erreur.
					case 'error' :
						console.log(r.message);
						_alertError(r.message);
						break;

					// Succès.
					case 'success' :
						$('#diaporama_sidebar_stats_favorites span').text(r.favorites_short);
						if (typeof favorites_update == 'function' && diaporama_item_id == item_id)
						{
							favorites_update(action, r);
						}
						_getData(false, q.match(/\/user-favorites\//) ? false : true,
							_currentPosition, _currentPosition);
						break;
				}
			}, 'json');
		});
	}

	/**
	 * Gestion des options.
	 *
	 * @return void
	 */
	function _setOptions()
	{
		// Options.
		if (typeof o == 'object')
		{
			for (var name in o)
			{
				_options[name] = o[name];
			}
		}

		// Sur mobile, on change certaines options.
		if (_isMobile())
		{
			_options.fullScreen = _options.fullScreenMobile;
		}

		// Préférences de l'utilisateur.
		try
		{
			var cookie_value = document.cookie
				.replace(/(?:(?:^|.*;\s*)igal_diaporama\s*\=\s*([^;]*).*$)|^.*$/, '$1');
			cookie_value = JSON.parse(cookie_value);
			if (typeof cookie_value == 'object')
			{
				for (var name in cookie_value)
				{
					_options[name] = JSON.parse(cookie_value[name]);
				}
			}
		}
		catch (e) {}

		// Sur mobile, on change certaines options.
		if (_isMobile())
		{
			_options.autoDurationMax = 10;
			_options.autoDurationPrecision = 0.5;
			_options.carousel = false;
			_options.controlBars = true;
			_options.keyboard = false;
			_options.showInformations = false;
			_options.sidebarItemResize = false;
			_options.transitionDuration = 300;
			_options.transitionEffect = 'slideX';
			if (_options.autoDuration > _options.autoDurationMax)
			{
				_options.autoDuration = _options.autoDurationMax;
			}
		}

		// Informations à afficher.
		if (typeof _options.sidebarInformations == 'object')
		{
			for (var name in _options.sidebarInformations)
			{
				if (!_options.sidebarInformations[name])
				{
					$('li[data-info-name="' + name + '"] .diaporama_sidebar_content').hide();
				}
			}
		}

		// Option "Afficher le carrousel".
		$('#diaporama_carousel_option').click(function()
		{
			_options.carousel = !_options.carousel;
			if (_options.carousel)
			{
				_getData(false, false, 0, _carouselCurrentPosition);
				if ($('#diaporama_top').is(':visible'))
				{
					$('#diaporama_carousel').show();
				}
				_carouselChangeSizePosition();
				_changeCenterSizePosition();
			}
			else
			{
				$('#diaporama_carousel').hide();
				_changeCenterSizePosition();
			}
			_changeItemSizePosition(_currentPosition);
			_previousDiaporamaSize = _getDiaporamaSize();
			_savePrefs();
		});
		if (_options.carousel)
		{
			$('#diaporama_carousel').show();
			_changeCenterSizePosition();
		}
		else
		{
			$('#diaporama_carousel').hide();
		}
		$('#diaporama_carousel_option').prop('checked', _options.carousel ? true : false);
		_options.carouselThumbsSize = _options.carouselThumbsSize < 50
			? 50
			: _options.carouselThumbsSize;

		// Option "Effet de transition".
		$('#diaporama_transitions_effect').change(function()
		{
			_options.transitionEffect = $('#diaporama_transitions_effect option:selected').val();
			_savePrefs();
		});
		$('#diaporama_transitions_effect option[value=' + _options.transitionEffect + ']')
			.prop('selected', true);

		// Option "Durée de transition".
		$('#diaporama_transitions_duration').keyup(function()
		{
			_options.transitionDuration = $('#diaporama_transitions_duration').val();
			_savePrefs();
		});
		$('#diaporama_transitions_duration').val(_options.transitionDuration);

		// Option "Lecture automatique au lancement du diaporama".
		$('#diaporama_autostart').click(function()
		{
			_options.autoStart = !_options.autoStart;
			_savePrefs();
		});
		$('#diaporama_autostart').prop('checked', _options.autoStart ? true : false);

		// Option "Lecture en boucle du mode automatique".
		$('#diaporama_autoloop').click(function()
		{
			_options.autoLoop = !_options.autoLoop;
			if (_currentPosition == _itemsCount && !_options.autoLoop)
			{
				_icon('stop').click();
			}
			_savePrefs();
		});
		$('#diaporama_autoloop').prop('checked', _options.autoLoop ? true : false);

		// Option "Mode plein écran au lancement".
		$('#diaporama_fullscreen').click(function()
		{
			_options.fullScreen = !_options.fullScreen;
			if (_options.fullScreen)
			{
				_fullScreen();
			}
			else
			{
				_fullScreenExit();
			}
			_savePrefs();
		});
		$('#diaporama_fullscreen').prop('checked', _options.fullScreen ? true : false);

		// Option "Afficher les barres de contrôle".
		$('#diaporama_controlbars').click(function()
		{
			_options.controlBars = !_options.controlBars;
			_changeItemSizePosition(_currentPosition);
			_changeCenterSizePosition();
			_savePrefs();
		});
		$('#diaporama_controlbars').prop('checked', _options.controlBars ? true : false);

		// Option "Afficher le panneau des informations au lancement".
		$('#diaporama_show_informations').click(function()
		{
			_options.showInformations = !_options.showInformations;
			_savePrefs();
		});
		$('#diaporama_show_informations').prop('checked',
			_options.showInformations ? true : false);

		// Option "Afficher la description par dessus les photos".
		$('#diaporama_over_image_description').click(function()
		{
			_options.overImageDescription = !_options.overImageDescription;
			_changeOverImageTextDisplay();
			_savePrefs();
		});
		$('#diaporama_over_image_description').prop('checked',
			_options.overImageDescription ? true : false);

		// Option "Afficher le titre par dessus les photos".
		$('#diaporama_over_image_title').click(function()
		{
			_options.overImageTitle = !_options.overImageTitle;
			_changeOverImageTextDisplay();
			_savePrefs();
		});
		$('#diaporama_over_image_title').prop('checked',
			_options.overImageTitle ? true : false);
	}

	/**
	 * Gestion de la sélection.
	 *
	 * @return void
	 */
	function _setSelectionEvents()
	{
		_icon('selection').click(function()
		{
			if (_items[_currentPosition] === undefined)
			{
				return;
			}

			var action = $(this).hasClass('active') ? 'remove' : 'add';

			if ((action == 'add' && _items[_currentPosition].in_selection)
			|| ((action == 'remove' && !_items[_currentPosition].in_selection)))
			{
				return;
			}

			if (action == 'remove')
			{
				$(this).removeClass('active');
				$('#diaporama_selection').removeClass('in');
			}
			else
			{
				$(this).addClass('active');
				$('#diaporama_selection').addClass('in');
			}

			$.post(gallery_path + '/ajax.php',
			{
				anticsrf: anticsrf,
				section: 'selection',
				action: 'selection-' + action,
				id: [_items[_currentPosition].id]
			},
			function(r)
			{
				if (typeof r != 'object' || r === null)
				{
					return;
				}
				switch (r.status)
				{
					// Erreur.
					case 'error' :
						console.log(r.message);
						_alertError(r.message);
						break;

					// Succès.
					case 'success' :
						var dl = $('.thumbs_items dl[data-id="'
							+ _items[_currentPosition].id + '"]');
						if (action == 'add')
						{
							dl.addClass('selected').find('.selection_icon').html('&#xea52;');
						}
						else
						{
							dl.removeClass('selected').find('.selection_icon').html('&#xea53;');
						}
						if (typeof selection_update == 'function')
						{
							if (!$('#item').length)
							{
								selection_update(r.stats);
							}
							if ($('#item').length && diaporama_item_id == item_id)
							{
								selection_update(action, r);
							}
						}
						_getData(false, q.match(/\/selection\/?/) ? false : true,
							_currentPosition, _currentPosition);
						break;
				}
			}, 'json');
		});
	}

	/**
	 * Gestionnaires d'événements pour les panneaux latéraux.
	 *
	 * @return void
	 */
	function _setSidebarsEvents()
	{
		$('.diaporama_sidebar_icon').each(function()
		{
			var name = '[data-name="' + $(this).attr('data-name') + '"]';
			var icon = $('.diaporama_sidebar_icon' + name);
			var sidebar = $('.diaporama_sidebar' + name);

			// Affichage du panneau latéral.
			$(this).click(function()
			{
				// Verrous.
				if (_sidebarMoveLock || _itemTransitionLock)
				{
					return;
				}
				_sidebarMoveLock = true;

				var diaporama_sidebar_width = _options.sidebarItemResize
					? $('.diaporama_sidebar').outerWidth() : 0;
				var action;

				// On affiche le panneau latéral.
				if (sidebar.is(':hidden'))
				{
					action = 'show';
					var sidebar_visible = false;
					var show_sidebar = function()
					{
						var css = _changeItemSizePosition(_currentPosition,
							true, true, true, -diaporama_sidebar_width);
						var after_show = function()
						{
							_previousDiaporamaSize = _getDiaporamaSize();
							_changeButtonSwitch();
							_sidebarMoveLock = false;
						};
						_carouselChangeSizePosition(-diaporama_sidebar_width);
						_changeCenterSizePosition(-diaporama_sidebar_width);
						if (_options.sidebarShowDuration && _options.animate)
						{
							$('#diaporama_item_' + _currentPosition).animate(
								css, _options.sidebarShowDuration
							);
							sidebar.show('slide', { direction: 'right' },
								_options.sidebarShowDuration, after_show);
						}
						else
						{
							$('#diaporama_item_' + _currentPosition).css(css);
							sidebar.show();
							after_show();
						}
						icon.addClass('active');
						sidebar.find('.focus').each(function()
						{
							$(this).focus();
							if (($(this).is('input') && $(this).attr('type') == 'text')
							 || $(this).is('textarea'))
							{
								$(this)[0].setSelectionRange(0, 0);
							}
						});
					}

					// Si un panneau latéral est déjà affiché,
					// on le cache avant d'afficher le panneau courant.
					$('.diaporama_sidebar').each(function()
					{
						if ($(this).is(':visible'))
						{
							$('.diaporama_sidebar_icon[data-name="'
								+ $(this).attr('data-name') + '"]').removeClass('active');
							if (_options.sidebarShowDuration && _options.animate)
							{
								$(this).hide('slide', { direction: 'right' },
									_options.sidebarShowDuration, show_sidebar);
							}
							else
							{
								$(this).hide();
								show_sidebar();
							}
							sidebar_visible = true;
						}
					});

					if (!sidebar_visible)
					{
						show_sidebar();
					}
				}

				// On cache le panneau latéral.
				else
				{
					action = 'hide';
					var css = _changeItemSizePosition(_currentPosition,
						true, true, true, diaporama_sidebar_width);
					var after_hide = function()
					{
						_previousDiaporamaSize = _getDiaporamaSize();
						_changeButtonSwitch();
						_sidebarMoveLock = false;
					};
					_carouselChangeSizePosition(diaporama_sidebar_width);
					_changeCenterSizePosition(diaporama_sidebar_width);
					if (_options.sidebarShowDuration && _options.animate)
					{
						$('#diaporama_item_' + _currentPosition).animate(
							css, _options.sidebarShowDuration
						);
						sidebar.hide('slide', { direction: 'right' },
							_options.sidebarShowDuration, after_hide);
					}
					else
					{
						$('#diaporama_item_' + _currentPosition).css(css);
						sidebar.hide();
						after_hide();
					}
					$(this).removeClass('active').focus();
				}

				// Div pour l'icône de favori.
				var left = $(window).width() - (action == 'show' ? diaporama_sidebar_width : 0);
				if (_options.sidebarShowDuration && _options.animate)
				{
					$('#diaporama_favorite,#diaporama_selection').animate({left: left},
						_options.itemResizeDuration);
				}
				else
				{
					$('#diaporama_favorite,#diaporama_selection').css({left: left});
				}
			});
		});

		// Bouton de fermeture des panneaux latéraux.
		$('.diaporama_sidebar_close').click(function()
		{
			$('.diaporama_sidebar_icon[data-name="'
				+ $(this).parents('section').attr('data-name') + '"]').click();
		});

		// Informations.
		$('.diaporama_sidebar[data-name="informations"] h2 span').click(function()
		{
			var li = $(this).parents('li');
			var name = li.attr('data-info-name');
			var content = li.find('.diaporama_sidebar_content');
			if (content.is(':hidden'))
			{
				content.slideDown('fast');
				_options.sidebarInformations[name] = 1;
			}
			else
			{
				content.slideUp('fast');
				_options.sidebarInformations[name] = 0;
			}
			_savePrefs();
		});

		// Édition : enregistrement des modifications.
		$('.diaporama_sidebar[data-name="edit"] form').submit(function()
		{
			if ($('#diaporama_edit_title').val().trim() == ''
			 || $('#diaporama_edit_filename').val().trim() == '')
			{
				return false;
			}

			$(':focus').blur();

			$(this).find('input[type="submit"]')
				.prop('disabled', true)
				.after('<span id="diaporama_edit_loading"></span>');

			var edit = '.diaporama_sidebar[data-name="edit"]';
			var edit_report = function(type, text)
			{
				$('#diaporama_edit_loading').remove();

				var icon;
				var duration = 3000;
				switch (type)
				{
					case 'info' :
						icon = '&#xf06a;';
						break;

					case 'error' :
						icon = '&#xe904;';
						duration = 6000;
						break;

					case 'success' :
						icon = '&#xe905;';
						break;
				}

				var html = '<div style="opacity:0" id="diaporama_report">'
					+ '<span class="diaporama_report_'
					+ type + '"><span>' + icon + '</span>' + text + '</span></div>';

				$(edit + ' .diaporama_buttons input').animate({opacity: 0}, 250);
				$(edit + ' .diaporama_buttons').append(html);
				$('#diaporama_report').animate({opacity: 1}, 250, function()
				{
					setTimeout(function()
					{
						$('#diaporama_report').animate({opacity: 0}, 250,
							function(){$(this).remove()});
						$(edit + ' .diaporama_buttons *').animate({opacity: 1}, 250,
							() => $(edit + ' input[type="submit"]').prop('disabled', false));
					}, duration);
				});
			};

			_keyboardActive = true;

			$.post(gallery_path + '/ajax.php',
			{
				section: 'item-edit',
				anticsrf: anticsrf,
				item_id: _items[_currentPosition].id,
				title: $('#diaporama_edit_title').val(),
				filename: $('#diaporama_edit_filename').val(),
				description: $('#diaporama_edit_desc').val(),
				tags: $('#diaporama_edit_tags').val()
			},
			function(r)
			{
				if (r === null)
				{
					edit_report('error', 'Null error.');
					return;
				}
				switch (r.status)
				{
					case 'info' :
					case 'success' :
						edit_report(r.status, r.message);

						// Mise à jour des informations sur la page du fichier.
						if (typeof item_update == 'function' && diaporama_item_id == item_id)
						{
							item_update(r, true);
						}

						// Titre.
						$('#diaporama_edit_title').val($('<input>').html(r.title).text());
						$('#diaporama_breadcrumb .current').html(r.title);
						_items[_currentPosition].title = r.title;

						// URL dans le fil d'Ariane.
						$('#diaporama_breadcrumb .current').attr('href',
							change_urlname($('#diaporama_breadcrumb .current')
							.attr('href'), r.urlname));

						// Nom de fichier.
						if (_items[_currentPosition].filename != r.filename)
						{
							$('#diaporama_edit_filename')
								.val($('<input>').html(r.filename).text());
							$('#diaporama_tools a[data-name="download"]')
								.attr('href', r.download);
							var item = '#diaporama_item_' + _currentPosition;
							if (r.is_video)
							{
								$(item).each(function()
								{
									this.pause();
									$(this).find('source').attr('src', r.source);
									this.load();
								});
							}
							else
							{
								$(item).attr('src', r.source);
							}
							_items[_currentPosition].filename = r.filename;
						}

						// Description.
						$('#diaporama_edit_desc')
							.val($('<input>').html(r.description).text());
						if (r.description === null)
						{
							$('#diaporama_sidebar_description').hide();
						}
						else
						{
							$('#diaporama_sidebar_description p').html(r.description_formated);
							$('#diaporama_sidebar_description').show();
						}
						_items[_currentPosition].description = r.description;

						// Tags.
						$('#diaporama_edit_tags').val(r.tags_list.join(', '));
						if (r.tags.length)
						{
							var html = '<ul>';
							for (var tag in r.tags)
							{
								html += '<li><a href="' + r.tags[tag].link + '">'
									+ r.tags[tag].name + '</a></li>';
							}
							html += '</ul>';
							$('#diaporama_sidebar_tags .diaporama_sidebar_content')
								.html(html);
							$('#diaporama_sidebar_tags').show();
						}
						else
						{
							$('#diaporama_sidebar_tags').hide();
						}

						// Titre et description sur l'image.
						_changeOverImageTextDisplay();
						break;

					case 'error' :
						edit_report(r.status, r.message);
						console.log(r.message);
						break;

					default :
						edit_report('error', 'Unknown error.');
						break;
				}
			}, 'json')
			.fail(function(r)
			{
				edit_report('error', 'PHP error.');
				console.log(r.responseText);
			});

			return false;
		});
	}

	/**
	 * Gestion du swipe.
	 *
	 * @return void
	 */
	function _setSwipeEvents()
	{
		if (!_options.swipe)
		{
			return;
		}

		var animate_lock = false;

		function css(e, css, callback)
		{
			if (_options.animate)
			{
				e.animate(css, parseInt(_options.swipeDuration), 'swing', function()
				{
					if (callback)
					{
						callback();
					}
				});
				return;
			}

			e.css(css);
			if (callback)
			{
				callback();
			}
		}

		function start()
		{
			if (!$('#diaporama_switch').hasClass('fullsize') && !_autoActive)	
			{
				swipe(event);
			}
			return false;
		}

		function swipe(evt)
		{
			if (animate_lock || _itemTransitionLock)
			{
				return;
			}

			var s = _getDiaporamaSize();
			var move_left = 0;
			var start = _isMobile() && evt.changedTouches ? evt.changedTouches[0] : evt;

			var item_current = $('#diaporama_item_' + _currentPosition);
			if (!item_current.length)
			{
				return;
			}
			_swipeActive = true;
			_changeItemSizePosition(_currentPosition);
			var item_current_left = parseFloat(item_current.css('left'));
			$('#diaporama_inner').css('cursor', 'grab');
			item_current.css('cursor', 'grab');

			var item_prev = $('#diaporama_item_' + (_currentPosition - 1));
			var item_prev_left = 0, item_prev_left_initial = 0;
			if (item_prev.length)
			{
				_changeItemSizePosition(_currentPosition - 1);
				item_prev_left_initial = parseFloat(item_prev.css('left'));
				item_prev_left = item_prev_left_initial - s.availableWidth;
				item_prev.css({display: 'block', left: item_prev_left});
			}

			var item_next = $('#diaporama_item_' + (_currentPosition + 1));
			var item_next_left = 0, item_next_left_initial = 0;
			if (item_next.length)
			{
				_changeItemSizePosition(_currentPosition + 1);
				item_next_left_initial = parseFloat(item_next.css('left'));
				item_next_left = item_next_left_initial + s.availableWidth;
				item_next.css({display: 'block', left: item_next_left});
			}

			function change()
			{
				_getData(false, false, _currentPosition, _currentPosition);
				_changeItemInfos();
				_changeButtonsNavigation();
				_changeButtonSwitch();
				_swipeActive = false;
				animate_lock = false;
			}

			function move(e)
			{
				$('#diaporama_item video').each(function(){this.pause()});

				var move;
				if (_isMobile())
				{
					if (!e.changedTouches)
					{
						return;
					}
					move = e.changedTouches[0];
				}
				else
				{
					move = e;
				}

				move_left = start.pageX - move.pageX;

				item_current.css('left', item_current_left - move_left);

				if (item_prev.length)
				{
					item_prev.css('left', item_prev_left - move_left);
				}
				if (item_next.length)
				{
					item_next.css('left', item_next_left - move_left);
				}

				return false;
			}

			function end()
			{
				$(document).unbind('mousemove mouseup');
				document.querySelector('#diaporama_inner').removeEventListener('touchmove', move);
				document.querySelector('#diaporama_inner').removeEventListener('touchend', end);
				$('#diaporama_inner').css('cursor', 'default');
				item_current.css('cursor', 'default');

				animate_lock = true;

				if (move_left > _options.swipeDistance && item_next.length)
				{
					css(item_prev, {left: (item_prev_left_initial - (s.availableWidth * 2))});
					css(item_current, {left: (item_current_left - s.availableWidth)},
						function()
						{
							item_prev.css('display', 'none');
							item_current.css('display', 'none');
							_currentPosition++;
							change();
						});
					css(item_next, {left: item_next_left_initial});

					return false;
				}
				if (move_left < -_options.swipeDistance && item_prev.length)
				{
					css(item_prev, {left: item_prev_left_initial});
					css(item_current, {left: (item_current_left + s.availableWidth)},
						function()
						{
							item_next.css('display', 'none');
							item_current.css('display', 'none');
							_currentPosition--;
							change();
						});
					css(item_next, {left: (item_next_left_initial + (s.availableWidth * 2))});

					return false;
				}

				css(item_current, {left: item_current_left});
				if (item_prev.length)
				{
					css(item_prev, {left: item_prev_left},
						() => item_prev.css('display', 'none'));
				}
				if (item_next.length)
				{
					css(item_next, {left: item_next_left},
						() => item_next.css('display', 'none'));
				}

				if (_options.animate)
				{
					setTimeout(function()
					{
						_swipeActive = false;
						animate_lock = false;
					},
					_options.swipeDuration);
				}
				else
				{
					_swipeActive = false;
					animate_lock = false;
				}

				return false;
			}

			$(document).mousemove(move);
			$(document).mouseup(end);
			document.querySelector('#diaporama_inner').addEventListener('touchmove', move);
			document.querySelector('#diaporama_inner').addEventListener('touchend', end);
		}

		$('#diaporama_inner').mousedown(start);
		document.querySelector('#diaporama_inner').addEventListener('touchstart', start);
	}

	/**
	 * Événements pour le zoom à la souris.
	 *
	 * @return void
	 */
	function _setZoomEvents()
	{
		document.querySelector('#diaporama_inner').onwheel = function(e)
		{
			_zoom = {};

			// Pas de zoom si le swipe est actif,
			// si l'option de zoom est désactivée
			// ou si le diaporama est fermé.
			if (_swipeActive || !_options.zoom || $('#diaporama').is(':hidden'))
			{
				return;
			}

			// Pas de zoom pour les vidéos ?
			if (_items[_currentPosition].is_video && !_options.zoomVideo)
			{
				return;
			}

			var s = _getDiaporamaSize();
			var item = $('#diaporama_item_' + _currentPosition);

			// Détermine si le pointeur se trouve sur le fichier.
			var cursor_item_in = e.clientX >= item.offset().left
				&& e.clientX <= (item.offset().left + item.width())
				&& e.clientY >= item.offset().top
				&& e.clientY < (item.offset().top + item.height());

			// Nouvelle taille (zoom).
			if (e.deltaY < 0)
			{
				_zoom.width = item.width() * _options.zoomFactor;
				_zoom.height = item.height() * _options.zoomFactor;
			}

			// Nouvelle taille (dézoom).
			else
			{
				_zoom.width = item.width() / _options.zoomFactor;
				_zoom.height = item.height() / _options.zoomFactor;
			}

			// Limitation du zoom.
			if (_options.zoomLimit == 100
			&& (_zoom.width > _items[_currentPosition].width_resized
			|| _zoom.height > _items[_currentPosition].height_resized))
			{
				_realsize = true;
				_zoom.width = _items[_currentPosition].width_resized;
				_zoom.height = _items[_currentPosition].height_resized;
			}
			if (_options.zoomLimit)
			{
				if (((_zoom.width / _items[_currentPosition].width_resized) * 100)
				> _options.zoomLimit)
				{
					_zoom.width = _items[_currentPosition].width_resized
						* (_options.zoomLimit / 100);
					_zoom.height = _items[_currentPosition].height_resized
						* (_options.zoomLimit / 100);
				}
			}

			// On évite d'avoir une taille plus petite
			// que la taille redimensionnée initiale.
			if (_zoom.width < item.attr('data-contain-width')
			 || _zoom.height < item.attr('data-contain-height'))
			{
				_zoom = {};
				_realsize = false;
				_changeItemSizePosition(_currentPosition, false);
				return;
			}

			// Position.
			var diff_height = _zoom.height - item.height();
			var diff_width = _zoom.width - item.width();
			var pos_ratio_height = 0.5;
			var pos_ratio_width = 0.5;
			if (_options.zoomZone && cursor_item_in)
			{
				pos_ratio_width = 1 / (item.width() / (e.clientX - item.offset().left));
				pos_ratio_height = 1 / (item.height() / (e.clientY - item.offset().top));
			}
			_zoom.top = item.offset().top - (diff_height * pos_ratio_height);
			_zoom.left = item.offset().left - (diff_width * pos_ratio_width);

			// Correction de la position (horizontale).
			if (_zoom.width < s.availableWidth)
			{
				_zoom.left = (s.availableWidth - _zoom.width) / 2;
			}
			else if ((_zoom.left + _zoom.width) < s.availableWidth)
			{
				_zoom.left += s.availableWidth - (_zoom.left + _zoom.width);
			}
			else if (_zoom.left > 0)
			{
				_zoom.left = 0;
			}

			// Correction de la position (verticale).
			if (_zoom.height < s.availableHeight)
			{
				_zoom.top = ((s.availableHeight - _zoom.height) / 2) + s.barTopHeight;
			}
			else if ((_zoom.top + _zoom.height - s.barTopHeight) < s.availableHeight)
			{
				_zoom.top += s.availableHeight
					- (_zoom.top + _zoom.height - s.barTopHeight);
			}
			else if (_zoom.top > s.barTopHeight)
			{
				_zoom.top = s.barTopHeight;
			}

			// Application des règles CSS.
			_zoom.width = Math.round(_zoom.width);
			_zoom.height = Math.round(_zoom.height);
			item.css({
				top: _zoom.top,
				left: _zoom.left,
				width: _zoom.width,
				height: _zoom.height
			});

			// Gestion du bouton de switch.
			_changeButtonSwitch();
		};
	}
};